from random import random

import anndata
import genomicranges
from biocframe import BiocFrame
import biocutils
import numpy as np
import pandas as pd
from mudata import MuData

import singlecellexperiment
from singlecellexperiment.SingleCellExperiment import SingleCellExperiment

import pytest

__author__ = "jkanche"
__copyright__ = "jkanche"
__license__ = "MIT"


def test_combine_rows(experiments):
    combined = biocutils.combine_rows(experiments.se_unnamed, experiments.se_unnamed_2)
    assert combined is not None
    assert isinstance(combined, SingleCellExperiment)
    assert len(combined.alternative_experiments) == 0
    assert len(combined.row_data["A"]) == 200

    combined2 = experiments.se_unnamed.combine_rows(experiments.se_unnamed_2)
    assert combined2 is not None
    assert isinstance(combined2, SingleCellExperiment)
    assert len(combined2.alternative_experiments) == 0
    assert len(combined2.row_data["A"]) == 200

def test_relaxed_combine_rows(experiments):
    ncols = 10
    nrows = 100
    test2 = experiments.se_unnamed_2.set_assays(
        {
            "counts": np.random.poisson(lam=10, size=(nrows, ncols)),
            "normalized": np.random.normal(size=(nrows, ncols)),
        },
        in_place=False,
    )

    with pytest.raises(Exception):
        combined = biocutils.combine_rows(experiments.se_unnamed, test2)

    combined = biocutils.relaxed_combine_rows(experiments.se_unnamed, test2)
    assert combined is not None
    assert isinstance(combined, SingleCellExperiment)
    assert len(combined.alternative_experiments) == 0
    assert len(combined.row_data["A"]) == 200

    combined2 = experiments.se_unnamed.relaxed_combine_rows(test2)
    assert combined2 is not None
    assert isinstance(combined2, SingleCellExperiment)
    assert len(combined2.alternative_experiments) == 0
    assert len(combined2.row_data["A"]) == 200


def test_combine_with_alts(experiments):
    combined = biocutils.combine_rows(
        experiments.se_with_alts1, experiments.se_with_alts2
    )
    assert combined is not None
    assert isinstance(combined, SingleCellExperiment)
    assert len(combined.alternative_experiments) == 1
    assert len(combined.reduced_dim_names) == 1
    assert combined.reduced_dim_names == ["PCA"]
    assert len(combined.row_data["seqnames"]) == 6
